package com.loit.common.langfang;

import com.loit.common.langfang.dto.CommandManualDTO;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.common.utils.excel.ImportExcel;
import com.loit.common.utils.file.FileUtils;
import com.loit.common.utils.freemarker.FreeMarkerUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
public class GeneratorCommandManualScript {

    public static final String CASE_TYPE = "1";
    protected static String root_path = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\langfang";


    public static void main(String[] args) {

        try {

            String filePathStr = "F:\\9Git140\\loit-build-common\\loit-build-component\\loit-build-deploy-env\\src\\main\\resources\\井盖大小类扩展20230516.xlsx";

            File pdfFile = new File(filePathStr);
            FileInputStream fileInputStream = new FileInputStream(pdfFile);
            MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),

                    ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);
            ImportExcel ei = new ImportExcel(multipartFile, 2, 0);
            List<CommandManualDTO> commandManualDTOList = ei.getDataList(CommandManualDTO.class);

            if (ListUtil.isEmpty(commandManualDTOList)) {
                return;
            }

            List<CommandManualDTO> transformList = transform(commandManualDTOList);

            createBigTypeScript(transformList);

            createSmallTypeScript(transformList);


        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 对excel中的数据做转换:
     * 1、去除空数据
     * 2、填充大类小类
     * 3、填充A类B类C类处置时限一样的
     * 4、
     */
    private static List<CommandManualDTO> transform(List<CommandManualDTO> commandManualDTOList) {

        String bigTypeCode = null;
        String bigTypeName = null;
        String smallTypeCode = null;
        String smallTypeName = null;

        List<CommandManualDTO> resultList = new ArrayList();

        for (CommandManualDTO commandManual : commandManualDTOList) {
            if (StringUtils.isEmpty(commandManual.getManageProblem())) {
                continue;
            }


            if (StringUtils.isNotEmpty(commandManual.getBigTypeCode())) {
                bigTypeCode = commandManual.getBigTypeCode();
            } else {
                commandManual.setBigTypeCode(bigTypeCode);
            }


            if (StringUtils.isNotEmpty(commandManual.getBigTypeName())) {
                bigTypeName = commandManual.getBigTypeName();
            } else {
                commandManual.setBigTypeName(bigTypeName);
            }


            if (StringUtils.isNotEmpty(commandManual.getSmallTypeCode())) {
                smallTypeCode = commandManual.getSmallTypeCode();
            } else {
                commandManual.setSmallTypeCode(smallTypeCode);
            }

            if (StringUtils.isNotEmpty(commandManual.getSmallTypeName())) {
                smallTypeName = commandManual.getSmallTypeName();
            } else {
                commandManual.setSmallTypeName(smallTypeName);
            }


            String atype = commandManual.getAtype();
            if (StringUtils.isEmpty(commandManual.getBtype())) {
                commandManual.setBtype(atype);
            }

            if (StringUtils.isEmpty(commandManual.getCtype())) {
                commandManual.setCtype(atype);
            }

            commandManual.setBigAndSmallCode(commandManual.getBigTypeCode() + "|"+ commandManual.getSmallTypeCode());



            commandManual.setBigTypeCodeInt(Integer.valueOf(commandManual.getBigTypeCode()));
            commandManual.setSmallTypeCodeInt(Integer.valueOf(commandManual.getSmallTypeCode()));

            resultList.add(commandManual);

        }

        return resultList;
    }


    /**
     * 生成大类脚本
     *
     * @throws IOException
     */
    private static void createBigTypeScript(List<CommandManualDTO> commandManualList) {


        List<CommandManualDTO> bitTypeList = commandManualList.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                new TreeSet<>(Comparator.comparing(CommandManualDTO::getBigTypeCode))), ArrayList::new));

        if (bitTypeList == null || bitTypeList.isEmpty()) {
            return;
        }

        for (CommandManualDTO commandManualDTO : bitTypeList) {
            Map model = new HashMap();
            model.put("bigTypeCode", commandManualDTO.getBigTypeCode());
            model.put("bigTypeName", commandManualDTO.getBigTypeName());
            model.put("caseType", CASE_TYPE);

            String configFilePath = root_path + "\\bigTypeScript.sql";
            String jobConfig = FreeMarkerUtils.process("langfang" + "\\bigTypeScript.ftl", model);
            FileUtils.append(configFilePath, jobConfig);
        }
    }


    /**
     * 生成小类脚本
     *
     * @throws IOException
     */
    private static void createSmallTypeScript(List<CommandManualDTO> commandManualList) {


        List<CommandManualDTO> smallTypeList = commandManualList.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() ->
                new TreeSet<>(Comparator.comparing(CommandManualDTO::getBigAndSmallCode))), ArrayList::new));

        if (smallTypeList == null || smallTypeList.isEmpty()) {
            return;
        }

        smallTypeList.sort(Comparator.comparing(CommandManualDTO::getBigTypeCodeInt)
                .thenComparing(CommandManualDTO::getSmallTypeCodeInt));

        for (CommandManualDTO commandManualDTO : smallTypeList) {
            Map model = new HashMap();
            model.put("bigTypeCode", commandManualDTO.getBigTypeCode());
            model.put("bigTypeName", commandManualDTO.getBigTypeName());
            model.put("smallTypeCode", commandManualDTO.getSmallTypeCode());
            model.put("smallTypeName", commandManualDTO.getSmallTypeName());

            model.put("caseType", CASE_TYPE);

            String configFilePath = root_path + "\\smallTypeScript.sql";
            String jobConfig = FreeMarkerUtils.process("langfang" + "\\smallTypeScript.ftl", model);
            FileUtils.append(configFilePath, jobConfig);
        }
    }

}
